package com.javacodegeeks.ultimate.aws.eb;

import com.amazonaws.auth.AWSCredentials;
import com.amazonaws.auth.AWSCredentialsProvider;
import com.amazonaws.client.builder.AwsClientBuilder;
import com.amazonaws.regions.Regions;
import com.amazonaws.services.dynamodbv2.AmazonDynamoDB;
import com.amazonaws.services.dynamodbv2.AmazonDynamoDBClientBuilder;
import com.amazonaws.services.dynamodbv2.document.DynamoDB;
import com.amazonaws.services.dynamodbv2.document.Table;
import com.amazonaws.services.dynamodbv2.model.*;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Response;
import java.util.ArrayList;
import java.util.List;

@Path("/db")
public class DbResource {

    private static final Log LOGGER = LogFactory.getLog(DbResource.class);

    @GET
    @Produces("text/json")
    @Path("/remove-all-tables")
    public Response removeAllTables() {
        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("Removing all tables.");
        }
        AmazonDynamoDB dynamoDB = createClient();
        ListTablesResult listTablesResult = dynamoDB.listTables();
        List<String> tableNames = listTablesResult.getTableNames();
        for (String table : tableNames) {
            dynamoDB.deleteTable(table);
        }
        return Response.status(200).entity(tableNames).build();
    }

    @GET
    @Produces("text/json")
    @Path("/create-all-tables")
    public Response createAllTables() {
        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("Creating all tables.");
        }
        AmazonDynamoDB dynamoDB = createClient();
        List<String> tableNames = new ArrayList<String>();
        createTable(dynamoDB, tableNames);
        return Response.status(200).entity(tableNames).build();
    }

    private void createTable(AmazonDynamoDB amazonDynamoDB, List<String> tableNames) {
        List<AttributeDefinition> attributeDefinitions = new ArrayList<AttributeDefinition>();
        attributeDefinitions.add(new AttributeDefinition().withAttributeName("author").withAttributeType("S"));
        List<KeySchemaElement> keySchema = new ArrayList<KeySchemaElement>();
        keySchema.add(new KeySchemaElement().withAttributeName("author").withKeyType(KeyType.HASH));
        CreateTableRequest request = new CreateTableRequest()
                .withTableName("tutorials")
                .withKeySchema(keySchema)
                .withAttributeDefinitions(attributeDefinitions)
                .withProvisionedThroughput(
                        new ProvisionedThroughput()
                                .withReadCapacityUnits(1L)
                                .withWriteCapacityUnits(1L));
        DynamoDB dynamoDB = new DynamoDB(amazonDynamoDB);
        Table table = dynamoDB.createTable(request);
        try {
            table.waitForActive();
        } catch (InterruptedException e) {
            LOGGER.error("Failed to wait for table to become active: " + e.getLocalizedMessage(), e);
        }
        tableNames.add("tutorials");
    }

    @GET
    @Produces("text/json")
    @Path("/list-all-tables")
    public Response listAllTables() {
        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("Listing all tables.");
        }
        AmazonDynamoDB dynamoDB = createClient();
        ListTablesResult listTablesResult = dynamoDB.listTables();
        List<String> tableNames = listTablesResult.getTableNames();
        return Response.status(200).entity(tableNames).build();
    }

    public static AmazonDynamoDB createClient() {
        String property = System.getProperty("tutorial.dynamodb.local-endpoint");
        Boolean localEndpoint = Boolean.valueOf(property);
        if (!localEndpoint) {
            return AmazonDynamoDBClientBuilder
                    .standard()
                    .withRegion(Regions.EU_CENTRAL_1)
                    .build();
        } else {
            LOGGER.debug("Using local endpoint.");
            return AmazonDynamoDBClientBuilder.standard()
                    .withEndpointConfiguration(
                            new AwsClientBuilder.EndpointConfiguration("http://localhost:8000", "eu-central-1"))
                    .withCredentials(new AWSCredentialsProvider() {
                        @Override
                        public AWSCredentials getCredentials() {
                            return new AWSCredentials() {
                                @Override
                                public String getAWSAccessKeyId() {
                                    return "dummy";
                                }

                                @Override
                                public String getAWSSecretKey() {
                                    return "dummy";
                                }
                            };
                        }

                        @Override
                        public void refresh() {

                        }
                    })
                    .build();
        }
    }
}
