package com.javacodegeeks.ultimate.aws.eb;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Response;
import java.sql.*;
import java.util.ArrayList;
import java.util.List;

@Path("/rds")
public class RdsResource {

    private static final Log LOGGER = LogFactory.getLog(RdsResource.class);

    @GET
    @Produces("text/json")
    @Path("/remove-all-tables")
    public Response removeAllTables() {
        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("Removing all tables.");
        }
        List<String> tableNames = new ArrayList<>();
        List<String> tables;
        try {
            tables = listAllTablesIntern();
        } catch (SQLException e) {
            return Response.status(500).entity("Listing all tables failed: " + e.getLocalizedMessage()).build();
        }
        for (String table : tables) {
            try (Connection connection = createConnection()) {
                if (connection != null) {
                    try (Statement stmt = connection.createStatement()) {
                        stmt.executeUpdate("drop table " + table);
                        tableNames.add(table);
                    }
                }
            } catch (SQLException e) {
                LOGGER.error("Removing all tables failed: " + e.getLocalizedMessage(), e);
                return Response.status(500).entity("Removing all tables failed: " + e.getLocalizedMessage()).build();
            }
        }
        return Response.status(200).entity(tableNames).build();
    }

    @GET
    @Produces("text/json")
    @Path("/create-all-tables")
    public Response createAllTables() {
        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("Creating all tables.");
        }
        List<String> tableNames = new ArrayList<>();
        try (Connection connection = createConnection()) {
            if (connection != null) {
                try (Statement stmt = connection.createStatement()) {
                    stmt.executeUpdate("create table tutorials (" +
                            "id int not null auto_increment, " +
                            "author varchar(30) not null," +
                            "title varchar(30) not null," +
                            "primary key (id)" +
                            ")");
                    tableNames.add("tutorials");
                }
            }
        } catch (SQLException e) {
            LOGGER.error("Creating tables failed: " + e.getLocalizedMessage(), e);
            return Response.status(500).entity("Creating tables failed: " + e.getLocalizedMessage()).build();
        }
        return Response.status(200).entity(tableNames).build();
    }

    @GET
    @Produces("text/json")
    @Path("/list-all-tables")
    public Response listAllTables() {
        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("Listing all tables.");
        }
        List<String> tableNames = null;
        try {
            tableNames = listAllTablesIntern();
        } catch (SQLException e) {
            return Response.status(500).entity("Listing all tables failed: " + e.getLocalizedMessage()).build();
        }
        return Response.status(200).entity(tableNames).build();
    }

    private List<String> listAllTablesIntern() throws SQLException {
        List<String> tableNames = new ArrayList<>();
        try (Connection connection = createConnection()) {
            if (connection != null) {
                try (Statement stmt = connection.createStatement()) {
                    ResultSet resultSet = stmt.executeQuery("show tables");
                    while (resultSet.next()) {
                        String tableName = resultSet.getString(1);
                        tableNames.add(tableName);
                    }
                }
            }
        }
        return tableNames;
    }

    public static Connection createConnection() throws SQLException {
        if (System.getProperty("RDS_HOSTNAME") != null) {
            try {
                Class.forName("com.mysql.jdbc.Driver");
                String dbName = System.getProperty("RDS_DB_NAME");
                String userName = System.getProperty("RDS_USERNAME");
                String password = System.getProperty("RDS_PASSWORD");
                String hostname = System.getProperty("RDS_HOSTNAME");
                String port = System.getProperty("RDS_PORT");
                String jdbcUrl = "jdbc:mysql://" + hostname + ":" + port + "/" + dbName;
                if (LOGGER.isDebugEnabled()) {
                    LOGGER.debug("Connecting to JDBC-URL: " + jdbcUrl);
                }
                Connection con = DriverManager.getConnection(jdbcUrl, userName, password);
                if (LOGGER.isDebugEnabled()) {
                    LOGGER.debug("Connection to JDBC-URL: " + jdbcUrl + " successful.");
                }
                return con;
            } catch (ClassNotFoundException e) {
                LOGGER.error("Could not load driver: " + e.getLocalizedMessage(), e);
            }
        }
        return null;
    }
}
